#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>

#ifdef HAVE_SYS_TIME_H
#	include <sys/time.h>
#	ifdef TIME_WITH_SYS_TIME
#		include <time.h>
#	endif
#else
#	include <time.h>
#endif

#include <unistd.h>
#include "common.h"

#ifndef VERSION
#define VERSION "v?.?.?"
#endif

struct vhostentry *hockum;
char *myself;
char conffile[256];
confetti bncconf;
FILE *mylife;
char logbuf[1024];
int foreman;

extern int loadconf (char *fname, confetti * jr);
extern int ircproxy (confetti * jr);
extern int initproxy (confetti * jr);

int bnclog (confetti * jr, char *logbuff)
{
  static long tata;
  int p;
  char tmpa[40];

  if (jr->logf != 1)
    return 0;
  tata = time ((long *) 0);
  snprintf (tmpa, 40, "%s", ctime (&tata));
  for (p = 0; p < 40; p++)
  {
    switch (tmpa[p])
    {
      case '\n':
      case '\r':
      case '\0':
	{
	  tmpa[p] = '\0';
	  break;
	}
    }
  }
  fprintf (jr->logfile, "%s: %s\n", tmpa, logbuff);
  fflush (jr->logfile);
  return 0;
}

void bnckill (int reason)
{
  char reply[1024];

  strcpy (reply, "Unknown error");
  switch (reason)
  {
    case CONFNOTFOUND:
      {
	strcpy (reply, "The config file (");
	strncat (reply, conffile, 1009);
	strcat (reply, ") was not found");
	break;
      }
    case CONFREQNOTHAPPY:
      {
	strcpy (reply, "Configuration file did not satisfy all requirments");
	break;
      }
    case FATALITY:
      {
	strcpy (reply, "Fatal error such as lack of ram occured");
	break;
      }
    case SOCKERR:
      {
	strcpy (reply, "Unable to open socket");
	break;
      }
    case BINDERR:
      {
	strcpy (reply, "Unable to bind to socket");
	break;
      }
    case LISTENERR:
      {
	strcpy (reply, "Unable to open socket for listen");
	break;
      }
    case BACKGROUND:
      {
	strcpy (reply, "Successfully went into the background");
	break;
      }
    case SELECTERR:
      {
	strcpy (reply, "Select call returned in error");
	break;
      }
    case DOWNER:
      {
	strcpy (reply, "Shut down by Supervisor");
	break;
      }
    case KILLED:
      {
	strcpy (reply, "I wuz shot down!");
	break;
      }
  }
  if (foreman)
    printf ("Exit %s{%i} :%s.\n", myself, reason, reply);
  snprintf (logbuf, 1024, "Exit %s{%i} :%s.\n", myself, reason, reply);
  bnclog (&bncconf, logbuf);
  exit (reason);
}
int main (int argc, char **argv)
{
  int tmps;
  char *avhd;
  char *pars;

  myself = argv[0];
  foreman = 1;
  if (argc > 1)
  {
    strncpy (conffile, argv[1], 256);
  }
  else
  {
    strncpy (conffile, argv[0], 256);
    strncat (conffile, ".conf", 256);
  }
  printf ("Irc Proxy " VERSION " GNU project (C) 1998-99\n");
  printf ("Coded by James Seter :bugs-> (Pharos@refract.com) or IRC pharos on efnet\n");
  printf ("--Using conf file %s\n", conffile);

  memset (&bncconf, 0, sizeof (bncconf));
  bncconf.cport = 6667;
  pars = strrchr (argv[0], '/');
  if (pars == NULL)
    pars = argv[0];
  else if (strlen (pars) > 1)
    pars++;

  strcpy (bncconf.pidfile, "./pid.");
  strncat (bncconf.pidfile, pars, 256);

  strcpy (bncconf.dpass, "-NONE-");
  bncconf.dpassf = 0;
  bncconf.identwd = 0;
  avhd = getenv ("IRC_HOST");
  if (avhd != NULL)
  {
    strncpy (bncconf.vhostdefault, avhd, 256);


  }
  tmps = loadconf (conffile, &bncconf);
  if(tmps)
  {
    bnckill (tmps);
  }
  printf ("--Configuration:\n");
  printf ("    Daemon port......:%u\n    Maxusers.........:%u\n    Default conn port:%u\n    Pid File.........:%s\n",
	bncconf.dport, bncconf.maxusers, bncconf.cport, bncconf.pidfile);
  if (bncconf.vhostdefault[0] == '\0')
  {
    printf ("    Vhost Default....:-SYSTEM DEFAULT-\n");
  }
  else
  {
    printf ("    Vhost Default....:%s\n", bncconf.vhostdefault);
  }
  if (bncconf.vhostlist != NULL)
  {
    hockum = bncconf.vhostlist;
    while (hockum != NULL)
    {
      printf ("    Vhost entry......:%s\n", hockum->vhost);
      hockum = hockum->next;
    }
  }
  tmps = initproxy (&bncconf);
  if(tmps)
  {
    bnckill (tmps);
  }
  signal (SIGHUP, SIG_IGN);
  switch (tmps = fork ())
  {
    case -1:
      {
	bnckill (FORKERR);
      }
    case 0:
      {
	foreman = 0;		/* this is a child process printing should no longer be allowed */
	setsid ();
	break;
      }
    default:
      {
	printf ("    Process Id.......:%i\n", tmps);
	bnckill (BACKGROUND);

      }
  }
  if ((mylife = fopen (bncconf.pidfile, "wb")) != NULL)
  {
    fprintf (mylife, "%i\n", getpid ());
    fclose (mylife);
  }
  snprintf (logbuf, 1024, "BNC started. pid %i", getpid ());
  bnclog (&bncconf, logbuf);
  tmps = ircproxy (&bncconf);
  if(tmps)
  {
    bnckill (tmps);
  }
  return 0;
}
